class ReadPair:
    def __init__(self, read_id, first, second):
        
        self.first = first
        self.second = second
            
        self.read_id = read_id
     
    def get_center(self):
        return (self.first.reference_start + self.second.reference_start + self.second.query_alignment_length) / 2
    
    def get_width(self):
        return self.second.reference_start + self.second.query_alignment_length - self.first.reference_start
           
    def get_left_pos(self):
        return self.first.reference_start
    
    def get_right_pos(self):       
        return self.second.reference_start + self.second.query_alignment_length
    
    def get_insert_size(self):
        return self.second.reference_start - self.first.reference_start
    
    def __str__(self):
        return f"{self.read_id} {self.get_center()} { self.first} {self.second}"
    
    def was_reversed(self):
        return self.was_reversed
    
    def reads_overlap(self):
        return True if self.first.mpos < self.first.aend else False



import sys
from math import log10
from rheos_common.util.common import *

class Locus(object):
    MIN_GL = sys.float_info.min

    def __init__(self, pos, ref, alt, gl_ref, gl_het, gl_hom, history):
        self.pos = pos
        self.ref = ref
        self.alt = alt
        self.gl_ref = gl_ref
        self.gl_het = gl_het
        self.gl_hom = gl_hom
        self.history = history
        # Read Depth
        self.dp = 0
        # Reference observation count
        self.ro = 0
        # Sum of reference observation qualities
        self.qr = 0
        self.ao = 0
        # Sum of alternate observation qualities
        self.qa = 0

    def add_to_history(self, new_val):
        self.history.append(new_val)

    def get_gls(self):
        return self.gl_ref, self.gl_het, self.gl_hom

    def get_log_gls(self):
        return log10(self.gl_ref), log10(self.gl_het), log10(self.gl_hom)

    def get_pl(self):
        vals = (int(to_phred(self.gl_ref)), int(to_phred(self.gl_het)), int(to_phred(self.gl_hom)))
        return tuple(map(lambda x: x - min(vals), vals))

    def get_gq(self):
        pl = self.get_pl()
        if pl[1] == 0:
            return pl[2]
        elif pl[2] == 0:
            return pl[1]
        else:
            print("Something went wrong in PL calculation")
            exit(1)

    def get_norm_gls(self):
        norm_const = self.gl_ref + self.gl_het + self.gl_hom
        return self.gl_ref / norm_const, self.gl_het / norm_const, self.gl_hom / norm_const

    def update_gls_to_min_if_zero(self):
        self.gl_hom = self.min_gl_if_zero(self.gl_hom)
        self.gl_het = self.min_gl_if_zero(self.gl_het)
        self.gl_ref = self.min_gl_if_zero(self.gl_ref)

    @classmethod
    def min_gl_if_zero(val):
        if val == 0:
            return Locus.MIN_GL
        else:
            return val




class VariantCandidate:
    def __init__(self, read_pair, var_type):
        self.supporting_reads = []
        self.supporting_reads.append(read_pair)
        
        self.left_pos = read_pair.get_left_pos()
        self.inner_left = read_pair.get_left_pos()
        self.right_pos = read_pair.get_right_pos()
        self.inner_right = read_pair.get_right_pos()
        self.var_type = var_type
        
        if self.left_pos > self.right_pos:
            print(read_pair)
            
        self.center = read_pair.get_center()
        
        self.width = read_pair.get_width()
        
        self.depth = 1
    
    def add_read_pair(self, read_pair):
        self.supporting_reads.append(read_pair)
        self.depth += 1
        self.left_pos = min(self.left_pos, read_pair.get_left_pos())
        self.inner_left = max(self.inner_left, read_pair.get_left_pos())
        
        self.right_pos = max(self.right_pos, read_pair.get_right_pos())
        self.inner_right = min(self.inner_right, read_pair.get_right_pos())
        self.center = (self.left_pos + self.right_pos) / 2
    
    def get_center(self):
        return self.center
    
    def get_width(self):
        return self.right_pos - self.left_pos
    
    def belongs(self, read_pair):
        the_center = read_pair.get_center()
        return the_center >= self.left_pos and the_center <= self.right_pos
        
    def overlaps(self, another):
        return (another.left_pos <= self.right_pos and another.left_pos >= self.left_pos) or (another.right_pos >= self.left_pos and another.right_pos <= self.right_pos)    
     
    def merge_with(self, another):
        for read_pair in another.supporting_reads:
            self.add_read_pair(read_pair)
            
    def inner_span_to_str(self):
        return "[inner-left:{} inner-right:{} span:{} depth:{}]".format(self.inner_left, self.inner_right, self.inner_right - self.inner_left, self.depth)   
            
    def get_inner_span(self):
        return self.inner_right - self.inner_left
    
    def __str__(self):
        ret = "{} ".format(self.var_type)
        for read in self.supporting_reads:
            ret += "[{},{}]\n".format(read.get_left_pos(), read.get_right_pos())
        return ret


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name="rheos_common",
    version="0.0.5",
    author="Sergei Yakneen",
    author_email="llevar@gmail.com",
    description="Classes common to various Rheos tools",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/llevar/rheos",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'sortedcontainers>=2.1.0'
    ]
)



from sortedcontainers import SortedDict
from rheos_common.model.variant_candidate import VariantCandidate


class VariantCandidateCollection:
    def __init__(self):
        self.variants = SortedDict()
        
    def add_variant_candidate(self, variant_candidate):
        self.variants[variant_candidate.get_center()] = variant_candidate     
     
    def add_read_pair(self, read_pair, var_type):
        
        if len(self.variants) == 0:
            new_var_can = VariantCandidate(read_pair, var_type)
            self.add_variant_candidate(new_var_can)
        else:
            center = read_pair.get_center()
            ind = self.variants.bisect(center)
            if ind >= 1:
                var_key = self.variants.iloc[ind-1]
                var_can = self.variants[var_key]
    
                if var_can.var_type == var_type:
                    if var_can.belongs(read_pair):
                        var_can.add_read_pair(read_pair)
                        return
                else:
                    print("Multiple variants at same location")
                    
            if ind < len(self.variants):
                var_key = self.variants.iloc[ind]
                var_can = self.variants[var_key]
    
                if var_can.belongs(read_pair):
                    var_can.add_read_pair(read_pair)
                    return
 
            new_var_can = VariantCandidate(read_pair, var_type)
            self.add_variant_candidate(new_var_can)
    
    def merge(self):
        first = True
        for key in self.variants:
            if first:
                cur_var = self.variants[key]
                first = False
                continue
            candidate = self.variants[key]
            if cur_var.overlaps(candidate):
                cur_var.merge_with(candidate)
                del self.variants[key]
            else:
                cur_var = candidate
                
    def purge(self):
        keys_to_purge = []
        for key in iter(self.variants):
            if self.variants[key].get_inner_span() < 0 or self.variants[key].depth <= 1:
               keys_to_purge.append(key)
        for key in keys_to_purge:
            del self.variants[key]